@@ -1,5 +1,5 @@
 /* Tcl/Tk command definitions for Insight - Breakpoints.
-   Copyright 2001 Free Software Foundation, Inc.
+   Copyright 2001, 2002 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -24,7 +24,6 @@
 #include "linespec.h"
 #include "breakpoint.h"
 #include "tracepoint.h"
-#include "gdb-events.h"
 
 #include <tcl.h>
 #include "gdbtk.h"
@@ -41,17 +40,17 @@ extern void report_error (void);
    They are also used in gdbtk-hooks.c */
 
 char *bptypes[] =
-{"none", "breakpoint", "hw breakpoint", "until",
- "finish", "watchpoint", "hw watchpoint",
- "read watchpoint", "acc watchpoint",
- "longjmp", "longjmp resume", "step resume",
- "sigtramp", "watchpoint scope",
- "call dummy", "shlib events", "catch load",
- "catch unload", "catch fork", "catch vfork",
- "catch exec", "catch catch", "catch throw"
-};
+  {"none", "breakpoint", "hw breakpoint", "until",
+   "finish", "watchpoint", "hw watchpoint",
+   "read watchpoint", "acc watchpoint",
+   "longjmp", "longjmp resume", "step resume",
+   "sigtramp", "watchpoint scope",
+   "call dummy", "shlib events", "catch load",
+   "catch unload", "catch fork", "catch vfork",
+   "catch exec", "catch catch", "catch throw"
+  };
 char *bpdisp[] =
-{"delete", "delstop", "disable", "donttouch"};
+  {"delete", "delstop", "disable", "donttouch"};
 
 /* Is this breakpoint interesting to a user interface? */
 #define BREAKPOINT_IS_INTERESTING(bp) \
@@ -116,6 +115,8 @@ static int gdb_trace_status (ClientData, Tcl_Interp *, int,
 			     Tcl_Obj * CONST[]);
 static int gdb_tracepoint_exists_command (ClientData, Tcl_Interp *,
 					  int, Tcl_Obj * CONST objv[]);
+static Tcl_Obj *get_breakpoint_commands (struct command_line *cmd);
+
 static int tracepoint_exists (char *args);
 
 /* Breakpoint/tracepoint events and related functions */
@@ -187,26 +188,22 @@ Gdbtk_Breakpoint_Init (Tcl_Interp *interp)
    disable,                     Disable it 
    donttouch                    Leave it alone 
    };
- */
+*/
 
 
 /* This implements the tcl command "gdb_find_bp_at_addr"
 
- * Tcl Arguments:
- *    addr:     address
- * Tcl Result:
- *    It returns a list of breakpoint numbers
- */
+* Tcl Arguments:
+*    addr:     address
+* Tcl Result:
+*    It returns a list of breakpoint numbers
+*/
 static int
-gdb_find_bp_at_addr (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
-
+gdb_find_bp_at_addr (ClientData clientData, Tcl_Interp *interp,
+		     int objc, Tcl_Obj *CONST objv[])
 {
   int i;
-  long addr;
+  CORE_ADDR addr;
 
   if (objc != 2)
     {
@@ -214,17 +211,13 @@ gdb_find_bp_at_addr (clientData, interp, objc, objv)
       return TCL_ERROR;
     }
 
-  if (Tcl_GetLongFromObj (interp, objv[1], &addr) == TCL_ERROR)
-    {
-      result_ptr->flags = GDBTK_IN_TCL_RESULT;
-      return TCL_ERROR;
-    }
+  addr = string_to_core_addr (Tcl_GetStringFromObj (objv[1], NULL));
 
   Tcl_SetListObj (result_ptr->obj_ptr, 0, NULL);
   for (i = 0; i < breakpoint_list_size; i++)
     {
       if (breakpoint_list[i] != NULL
-	  && breakpoint_list[i]->address == (CORE_ADDR) addr)
+	  && breakpoint_list[i]->address == addr)
 	Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
 				  Tcl_NewIntObj (i));
     }
@@ -234,23 +227,19 @@ gdb_find_bp_at_addr (clientData, interp, objc, objv)
 
 /* This implements the tcl command "gdb_find_bp_at_line"
 
- * Tcl Arguments:
- *    filename: the file in which to find the breakpoint
- *    line:     the line number for the breakpoint
- * Tcl Result:
- *    It returns a list of breakpoint numbers
- */
+* Tcl Arguments:
+*    filename: the file in which to find the breakpoint
+*    line:     the line number for the breakpoint
+* Tcl Result:
+*    It returns a list of breakpoint numbers
+*/
 static int
-gdb_find_bp_at_line (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_find_bp_at_line (ClientData clientData, Tcl_Interp *interp,
+		     int objc, Tcl_Obj *CONST objv[])
 
 {
   struct symtab *s;
-  int line;
-  int i;
+  int i, line;
 
   if (objc != 3)
     {
@@ -258,7 +247,7 @@ gdb_find_bp_at_line (clientData, interp, objc, objv)
       return TCL_ERROR;
     }
 
-  s = full_lookup_symtab (Tcl_GetStringFromObj (objv[1], NULL));
+  s = lookup_symtab (Tcl_GetStringFromObj (objv[1], NULL));
   if (s == NULL)
     return TCL_ERROR;
 
@@ -293,7 +282,6 @@ gdb_get_breakpoint_info (ClientData clientData, Tcl_Interp *interp, int objc,
 			 Tcl_Obj *CONST objv[])
 {
   struct symtab_and_line sal;
-  struct command_line *cmd;
   int bpnum;
   struct breakpoint *b;
   char *funcname, *filename;
@@ -315,10 +303,7 @@ gdb_get_breakpoint_info (ClientData clientData, Tcl_Interp *interp, int objc,
   b = (bpnum <= breakpoint_list_size ? breakpoint_list[bpnum] : NULL);
   if (!b || b->type != bp_breakpoint)
     {
-      char *err_buf;
-      xasprintf (&err_buf, "Breakpoint #%d does not exist.", bpnum);
-      Tcl_SetStringObj (result_ptr->obj_ptr, err_buf, -1);
-      free(err_buf);
+      gdbtk_set_result (interp, "Breakpoint #%d does not exist.", bpnum);
       return TCL_ERROR;
     }
 
@@ -349,11 +334,8 @@ gdb_get_breakpoint_info (ClientData clientData, Tcl_Interp *interp, int objc,
   Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
 			    Tcl_NewIntObj (b->ignore_count));
 
-  new_obj = Tcl_NewObj ();
-  for (cmd = b->commands; cmd; cmd = cmd->next)
-    Tcl_ListObjAppendElement (NULL, new_obj,
-			      Tcl_NewStringObj (cmd->line, -1));
-  Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr, new_obj);
+  Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
+			    get_breakpoint_commands (b->commands));
 
   Tcl_ListObjAppendElement (NULL, result_ptr->obj_ptr,
 			    Tcl_NewStringObj (b->cond_string, -1));
@@ -371,6 +353,75 @@ gdb_get_breakpoint_info (ClientData clientData, Tcl_Interp *interp, int objc,
   return TCL_OK;
 }
 
+/* Helper function for gdb_get_breakpoint_info, this function is
+   responsible for figuring out what to type at the "commands" command
+   in gdb's cli in order to get at the same command list passed here.
+
+   NOTE: cannot use sprintf_append_element_to_obj with anything from
+   gdb, since those things could contain unescaped sequences. */
+static Tcl_Obj *
+get_breakpoint_commands (struct command_line *cmd)
+{
+  Tcl_Obj *obj, *tmp;
+
+  obj = Tcl_NewObj ();
+  while (cmd != NULL)
+    {
+      switch (cmd->control_type)
+	{
+	case simple_control:
+	  /* A simple command. Just append it. */
+	  Tcl_ListObjAppendElement (NULL, obj,
+				    Tcl_NewStringObj (cmd->line, -1));
+	  break;
+
+	case break_control:
+	  /* A loop_break */
+	  sprintf_append_element_to_obj (obj, "loop_break");
+	  break;
+
+	case continue_control:
+	  /* A loop_continue */
+	  sprintf_append_element_to_obj (obj, "loop_continue");
+	  break;
+
+	case while_control:
+	  /* A while loop. Must append "end" to the end of it. */
+	  tmp = Tcl_NewStringObj ("while ", -1);
+	  Tcl_AppendToObj (tmp, cmd->line, -1);
+	  Tcl_ListObjAppendElement (NULL, obj, tmp);
+	  Tcl_ListObjAppendList (NULL, obj,
+				 get_breakpoint_commands (*cmd->body_list));
+	  sprintf_append_element_to_obj (obj, "end");
+	  break;
+
+	case if_control:
+	  /* An if statement. cmd->body_list[0] is the true part,
+	     cmd->body_list[1] contains the "else" (false) part. */
+	  tmp = Tcl_NewStringObj ("if ", -1);
+	  Tcl_AppendToObj (tmp, cmd->line, -1);
+	  Tcl_ListObjAppendElement (NULL, obj, tmp);
+	  Tcl_ListObjAppendList (NULL, obj,
+				 get_breakpoint_commands (cmd->body_list[0]));
+	  if (cmd->body_count == 2)
+	    {
+	      sprintf_append_element_to_obj (obj, "else");
+	      Tcl_ListObjAppendList (NULL, obj,
+				     get_breakpoint_commands(cmd->body_list[1]));
+	    }
+	  sprintf_append_element_to_obj (obj, "end");
+	  break;
+
+	case invalid_control:
+	  /* Something invalid. Just skip it. */
+	  break;
+	}
+
+      cmd = cmd->next;
+    }
+
+  return obj;
+}
 
 /* This implements the tcl command gdb_get_breakpoint_list
  * It builds up a list of the current breakpoints.
@@ -381,11 +432,8 @@ gdb_get_breakpoint_info (ClientData clientData, Tcl_Interp *interp, int objc,
  *    A list of breakpoint numbers.
  */
 static int
-gdb_get_breakpoint_list (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_breakpoint_list (ClientData clientData, Tcl_Interp *interp,
+			 int objc, Tcl_Obj *CONST objv[])
 {
   int i;
   Tcl_Obj *new_obj;
@@ -421,11 +469,8 @@ gdb_get_breakpoint_list (clientData, interp, objc, objv)
  *    The return value of the call to gdbtk_tcl_breakpoint.
  */
 static int
-gdb_set_bp (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_set_bp (ClientData clientData, Tcl_Interp *interp,
+	    int objc, Tcl_Obj *CONST objv[])
 {
   struct symtab_and_line sal;
   int line, thread = -1;
@@ -439,7 +484,7 @@ gdb_set_bp (clientData, interp, objc, objv)
       return TCL_ERROR;
     }
 
-  sal.symtab = full_lookup_symtab (Tcl_GetStringFromObj (objv[1], NULL));
+  sal.symtab = lookup_symtab (Tcl_GetStringFromObj (objv[1], NULL));
   if (sal.symtab == NULL)
     return TCL_ERROR;
 
@@ -450,19 +495,13 @@ gdb_set_bp (clientData, interp, objc, objv)
     }
 
   typestr = Tcl_GetStringFromObj (objv[3], NULL);
-  if (typestr == NULL)
-    {
-      result_ptr->flags = GDBTK_IN_TCL_RESULT;
-      return TCL_ERROR;
-    }
   if (strncmp (typestr, "temp", 4) == 0)
     disp = disp_del;
   else if (strncmp (typestr, "normal", 6) == 0)
     disp = disp_donttouch;
   else
     {
-      Tcl_SetStringObj (result_ptr->obj_ptr,
-			"type must be \"temp\" or \"normal\"", -1);
+      gdbtk_set_result (interp, "type must be \"temp\" or \"normal\"");
       return TCL_ERROR;
     }
 
@@ -488,7 +527,7 @@ gdb_set_bp (clientData, interp, objc, objv)
 
   /* FIXME: this won't work for duplicate basenames! */
   xasprintf (&buf, "%s:%d", basename (Tcl_GetStringFromObj (objv[1], NULL)),
-	   line);
+	     line);
   b->addr_string = xstrdup (buf);
   free(buf);
 
@@ -514,9 +553,9 @@ gdb_set_bp_addr (ClientData clientData, Tcl_Interp *interp, int objc,
 {
   struct symtab_and_line sal;
   int thread = -1;
-  long addr;
+  CORE_ADDR addr;
   struct breakpoint *b;
-  char *typestr, *buf;
+  char *saddr, *typestr;
   enum bpdisp disp;
 
   if (objc != 3 && objc != 4)
@@ -525,26 +564,17 @@ gdb_set_bp_addr (ClientData clientData, Tcl_Interp *interp, int objc,
       return TCL_ERROR;
     }
 
-  if (Tcl_GetLongFromObj (interp, objv[1], &addr) == TCL_ERROR)
-    {
-      result_ptr->flags = GDBTK_IN_TCL_RESULT;
-      return TCL_ERROR;
-    }
-
+  saddr = Tcl_GetStringFromObj (objv[1], NULL);
+  addr = string_to_core_addr (saddr);
+  
   typestr = Tcl_GetStringFromObj (objv[2], NULL);
-  if (typestr == NULL)
-    {
-      result_ptr->flags = GDBTK_IN_TCL_RESULT;
-      return TCL_ERROR;
-    }
   if (strncmp (typestr, "temp", 4) == 0)
     disp = disp_del;
   else if (strncmp (typestr, "normal", 6) == 0)
     disp = disp_donttouch;
   else
     {
-      Tcl_SetStringObj (result_ptr->obj_ptr,
-			"type must be \"temp\" or \"normal\"", -1);
+      gdbtk_set_result (interp, "type must be \"temp\" or \"normal\"");
       return TCL_ERROR;
     }
 
@@ -564,9 +594,7 @@ gdb_set_bp_addr (ClientData clientData, Tcl_Interp *interp, int objc,
   b->number = breakpoint_count;
   b->disposition = disp;
   b->thread = thread;
-
-  xasprintf (&buf, "*(0x%lx)", addr);
-  b->addr_string = xstrdup (buf);
+  b->addr_string = xstrdup (saddr);
 
   /* now send notification command back to GUI */
   breakpoint_create_event (b->number);
@@ -636,9 +664,7 @@ gdbtk_modify_breakpoint (int num)
  * On error, the error string is written to gdb_stdout.
  */
 static void
-breakpoint_notify (num, action)
-     int num;
-     const char *action;
+breakpoint_notify (int num, const char *action)
 {
   char *buf;
 
@@ -673,11 +699,8 @@ breakpoint_notify (num, action)
  */
 
 static int
-gdb_actions_command (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_actions_command (ClientData clientData, Tcl_Interp *interp,
+		     int objc, Tcl_Obj *CONST objv[])
 {
   struct tracepoint *tp;
   Tcl_Obj **actions;
@@ -742,11 +765,8 @@ gdb_actions_command (clientData, interp, objc, objv)
 }
 
 static int
-gdb_get_trace_frame_num (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_trace_frame_num (ClientData clientData, Tcl_Interp *interp,
+			 int objc, Tcl_Obj *CONST objv[])
 {
   if (objc != 1)
     {
@@ -784,14 +804,11 @@ gdb_get_tracepoint_info (ClientData clientData, Tcl_Interp *interp,
 
   ALL_TRACEPOINTS (tp)
     if (tp->number == tpnum)
-    break;
+      break;
 
   if (tp == NULL)
     {
-      char *buff;
-      xasprintf (&buff, "Tracepoint #%d does not exist", tpnum);
-      Tcl_SetStringObj (result_ptr->obj_ptr, buff, -1);
-      free(buff);
+      gdbtk_set_result (interp, "Tracepoint #%d does not exist", tpnum);
       return TCL_ERROR;
     }
 
@@ -841,11 +858,10 @@ gdb_get_tracepoint_info (ClientData clientData, Tcl_Interp *interp,
 
 /* return a list of all tracepoint numbers in interpreter */
 static int
-gdb_get_tracepoint_list (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_get_tracepoint_list (ClientData clientData,
+			 Tcl_Interp *interp,
+			 int objc,
+			 Tcl_Obj *CONST objv[])
 {
   struct tracepoint *tp;
 
@@ -859,11 +875,10 @@ gdb_get_tracepoint_list (clientData, interp, objc, objv)
 }
 
 static int
-gdb_trace_status (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_trace_status (ClientData clientData,
+		  Tcl_Interp *interp,
+		  int objc,
+		  Tcl_Obj *CONST objv[])
 {
   int result = 0;
 
@@ -896,17 +911,17 @@ tracepoint_exists (char *args)
 	  strcat (file, sals.sals[0].symtab->filename);
 
 	  ALL_TRACEPOINTS (tp)
-	  {
-	    if (tp->address == sals.sals[0].pc)
-	      result = tp->number;
+	    {
+	      if (tp->address == sals.sals[0].pc)
+		result = tp->number;
 #if 0
-	    /* Why is this here? This messes up assembly traces */
-	    else if (tp->source_file != NULL
-		     && strcmp (tp->source_file, file) == 0
-		     && sals.sals[0].line == tp->line_number)
-	      result = tp->number;
+	      /* Why is this here? This messes up assembly traces */
+	      else if (tp->source_file != NULL
+		       && strcmp (tp->source_file, file) == 0
+		       && sals.sals[0].line == tp->line_number)
+		result = tp->number;
 #endif
-	  }
+	    }
 	}
     }
   if (file != NULL)
@@ -915,11 +930,10 @@ tracepoint_exists (char *args)
 }
 
 static int
-gdb_tracepoint_exists_command (clientData, interp, objc, objv)
-     ClientData clientData;
-     Tcl_Interp *interp;
-     int objc;
-     Tcl_Obj *CONST objv[];
+gdb_tracepoint_exists_command (ClientData clientData,
+			       Tcl_Interp *interp,
+			       int objc,
+			       Tcl_Obj *CONST objv[])
 {
   char *args;
 
@@ -960,9 +974,7 @@ gdbtk_modify_tracepoint (int num)
 }
 
 static void
-tracepoint_notify (num, action)
-     int num;
-     const char *action;
+tracepoint_notify (int num, const char *action)
 {
   char *buf;
 
