@@ -1,5 +1,5 @@
 /* Startup code for Insight
-   Copyright 1994, 1995, 1996, 1997, 1998, 2001 
+   Copyright 1994, 1995, 1996, 1997, 1998, 2001, 2002
    Free Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
@@ -22,46 +22,42 @@
    Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
-#include "symtab.h"
 #include "inferior.h"
-#include "command.h"
-#include "bfd.h"
 #include "symfile.h"
 #include "objfiles.h"
-#include "target.h"
 #include "gdbcore.h"
 #include "tracepoint.h"
 #include "demangle.h"
 #include "version.h"
 #include "cli-out.h"
+#include "top.h"
+#include "annotate.h"
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 
-#include <sys/stat.h>
+/* tcl header files includes varargs.h unless HAS_STDARG is defined,
+   but gdb uses stdarg.h, so make sure HAS_STDARG is defined.  */
+#define HAS_STDARG 1
 
-#include <tcl.h>
-#include <tk.h>
-#include <itcl.h>
 #include <tix.h>
+#include <itcl.h>
 #include <itk.h>
 #include "guitcl.h"
 #include "gdbtk.h"
 
-#include <signal.h>
 #include <fcntl.h>
-#include "top.h"
+#include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <sys/time.h>
+#include <signal.h>
+
 #include "gdb_string.h"
 #include "dis-asm.h"
-#include <stdio.h>
 #include "gdbcmd.h"
 
-#include "annotate.h"
-#include <sys/time.h>
-
 #ifdef __CYGWIN32__
 #include <sys/cygwin.h>		/* for cygwin32_attach_handle_to_fd */
 #endif
@@ -74,24 +70,17 @@ static sigset_t nullsigmask;
 static struct sigaction act1, act2;
 static struct itimerval it_on, it_off;
 
-static void x_event_wrapper (int);
 static void
-x_event_wrapper (signo)
-     int signo;
+x_event_wrapper (int signo)
 {
   x_event (signo);
 }
 
- /*
-  * These two variables control the interaction with an external editor.
-  * If enable_external_editor is set at startup, BEFORE Gdbtk_Init is run
-  * then the Tcl variable of the same name will be set, and a command will
-  * called external_editor_command will be invoked to call out to the
-  * external editor.  We give a dummy version here to warn if it is not set.
-  */
-int enable_external_editor = 0;
-char *external_editor_command = "tk_dialog .warn-external \\\n\
-\"No command is specified.\nUse --tclcommand <tcl/file> or --external-editor <cmd> to specify a new command\" 0 Ok";
+/*
+ * This variable controls the interaction with an external editor.
+ */
+
+char *external_editor_command = NULL;
 
 extern int Tktable_Init (Tcl_Interp * interp);
 
@@ -139,23 +128,19 @@ int gdbtk_disable_fputs = 1;
 
 #if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION == 0
 char *
-TclpAlloc (size)
-     unsigned int size;
+TclpAlloc (unsigned int size)
 {
   return xmalloc (size);
 }
 
 char *
-TclpRealloc (ptr, size)
-     char *ptr;
-     unsigned int size;
+TclpRealloc (char *ptr, unsigned int size)
 {
   return xrealloc (ptr, size);
 }
 
 void
-TclpFree (ptr)
-     char *ptr;
+TclpFree (char *ptr)
 {
   free (ptr);
 }
@@ -176,13 +161,13 @@ void
 close_bfds ()
 {
   struct objfile *o;
-
+  
   ALL_OBJFILES (o)
-  {
-    if (o->obfd != NULL)
-      bfd_cache_close (o->obfd);
-  }
-
+    {
+      if (o->obfd != NULL)
+	bfd_cache_close (o->obfd);
+    }
+  
   if (exec_bfd != NULL)
     bfd_cache_close (exec_bfd);
 }
@@ -319,16 +304,14 @@ gdbtk_stop_timer ()
 /* Should this target use the timer? See comments before
    x_event for the logic behind all this. */
 static int
-target_should_use_timer (t)
-     struct target_ops *t;
+target_should_use_timer (struct target_ops *t)
 {
   return target_is_native (t);
 }
 
 /* Is T a native target? */
 int
-target_is_native (t)
-     struct target_ops *t;
+target_is_native (struct target_ops *t)
 {
   char *name = t->to_shortname;
 
@@ -344,8 +327,7 @@ target_is_native (t)
 /* gdbtk_init installs this function as a final cleanup.  */
 
 static void
-gdbtk_cleanup (dummy)
-     PTR dummy;
+gdbtk_cleanup (PTR dummy)
 {
   Tcl_Eval (gdbtk_interp, "gdbtk_cleanup");
   Tcl_Finalize ();
@@ -360,11 +342,13 @@ gdbtk_cleanup (dummy)
  */
 
 static void
-gdbtk_init (argv0)
-     char *argv0;
+gdbtk_init (char *argv0)
 {
   struct cleanup *old_chain;
   char *s;
+  int element_count;
+  char **exec_path;
+  CONST char *internal_exec_name;
 
   /* If there is no DISPLAY environment variable, Tk_Init below will fail,
      causing gdb to abort.  If instead we simply return here, gdb will
@@ -372,7 +356,10 @@ gdbtk_init (argv0)
 
 #ifndef _WIN32
   if (getenv ("DISPLAY") == NULL)
-    return;
+    {
+      init_ui_hook = NULL;
+      return;
+    }
 #endif
 
   old_chain = make_cleanup (cleanup_init, 0);
@@ -388,6 +375,106 @@ gdbtk_init (argv0)
   if (!gdbtk_interp)
     error ("Tcl_CreateInterp failed");
 
+  /* We need to check if we are being run from
+     a bin directory, if not then we may have to
+     set some environment variables. */
+
+  internal_exec_name = Tcl_GetNameOfExecutable ();
+
+  Tcl_SplitPath ((char *) internal_exec_name, &element_count, &exec_path);
+
+  if (strcmp (exec_path[element_count - 2], "bin") != 0)
+    {
+      /* We check to see if TCL_LIBRARY, TK_LIBRARY,
+	 ITCL_LIBRARY, ITK_LIBRARY, TIX_LIBRARY and maybe
+	 a couple other environment variables have been
+	 set (we don't want to override the User's settings).
+	 If the *_LIBRARY variable is not set, point it at
+	 the source directory. */
+
+      static char set_libs_path_script[] = "\
+	  set srcDir [file dirname [file dirname $env(TCL_LIBRARY)]];\n\
+\
+	  if {![info exists env(TK_LIBRARY)]} {\n\
+	      set env(TK_LIBRARY) [file join $srcDir tk library]\n\
+	  }\n\
+\
+	  if {![info exists env(ITCL_LIBRARY)]} {\n\
+	      set env(ITCL_LIBRARY) [file join $srcDir itcl itcl library]\n\
+	  }\n\
+\
+	  if {![info exists env(ITK_LIBRARY)]} {\n\
+	      set env(ITK_LIBRARY) [file join $srcDir itcl itk library]\n\
+	  }\n\
+\
+	  if {![info exists env(IWIDGETS_LIBRARY)]} {\n\
+	      set env(IWIDGETS_LIBRARY)\
+                     [file join $srcDir itcl iwidgets3.0.0 generic]\n\
+	  }\n\
+\
+	  if {![info exists env(TIX_LIBRARY)]} {\n\
+	      set env(TIX_LIBRARY) [file join $srcDir tix library]\n\
+	  }\n\
+\
+	  if {![info exists env(GDBTK_LIBRARY)]} {\n\
+	      set env(GDBTK_LIBRARY) [file join $srcDir gdb gdbtk library]\n\
+	  }\n\
+\
+          # Append the directory with the itcl pkg index\n\
+          if {[info exists env(TCLLIBPATH)]} {\n\
+            append env(TCLLIBPATH) :[file joing $srcDir itcl]\n\
+          } else {\n\
+            set env(TCLLIBPATH) [file join $srcDir itcl]\n\
+          }\n\
+\
+          # We also need to append the iwidgets library path.\n\
+          # Unfortunately, there is no IWIDGETS_LIBRARY.\n\
+          set IWIDGETS_LIBRARY [file join $srcDir itcl iwidgets3.0.0 generic]\n";
+
+      Tcl_Obj *commandObj;
+
+      /* Before we can run our script we must set TCL_LIBRARY. */
+      if (Tcl_GetVar2 (gdbtk_interp, "env", "TCL_LIBRARY", TCL_GLOBAL_ONLY) == NULL)
+	{
+	  int i, count;
+	  char *src_dir = SRC_DIR;
+	  char **src_path;
+	  char **lib_path;
+	  Tcl_DString lib_dstring;
+
+	  Tcl_DStringInit (&lib_dstring);
+
+#ifdef __CYGWIN__
+	  /* SRC_DIR from configure is a posix path. Tcl really needs a
+	     windows path. */
+	  src_dir = (char *) alloca (cygwin_posix_to_win32_path_list_buf_size (SRC_DIR));
+	  cygwin_posix_to_win32_path_list (SRC_DIR, src_dir);
+#endif
+	  Tcl_SplitPath (src_dir, &count, &src_path);
+
+	  /* Append tcl/library to src_dir (src_dir=/foo/bar/gdb) */
+	  lib_path = (char **) alloca ((count + 2) * sizeof (char *));
+	  for (i = 0; i < count - 1; i++)
+	    lib_path[i] = src_path[i];
+	  lib_path[i++] = "tcl";
+	  lib_path[i++] = "library";
+	  Tcl_JoinPath (i, lib_path, &lib_dstring);
+
+	  /* Set TCL_LIBRARY */
+	  Tcl_SetVar2 (gdbtk_interp, "env", "TCL_LIBRARY",
+		       Tcl_DStringValue (&lib_dstring) , TCL_GLOBAL_ONLY);
+	  Tcl_DStringFree (&lib_dstring);
+	  Tcl_Free ((char *) src_path);
+	}
+
+      commandObj = Tcl_NewStringObj (set_libs_path_script, -1);
+      Tcl_IncrRefCount (commandObj);
+      Tcl_EvalObj (gdbtk_interp, commandObj);
+      Tcl_DecrRefCount (commandObj);
+    }
+
+  Tcl_Free ((char *) exec_path);
+
   if (Tcl_Init (gdbtk_interp) != TCL_OK)
     error ("Tcl_Init failed: %s", gdbtk_interp->result);
 
@@ -467,7 +554,7 @@ gdbtk_init (argv0)
       error ("Gdbtk_Init failed: %s", gdbtk_interp->result);
     }
 
-  Tcl_StaticPackage (gdbtk_interp, "Gdbtk", Gdbtk_Init, NULL);
+  Tcl_StaticPackage (gdbtk_interp, "Insight", Gdbtk_Init, NULL);
 
   /* This adds all the hooks that call up from the bowels of gdb
    *  back into Tcl-land...
@@ -482,13 +569,16 @@ gdbtk_init (argv0)
 	   "Send a command directly into tk.");
 
   /*
-   * Set the variables for external editor:
+   * Set the variable for external editor:
    */
 
-  Tcl_SetVar (gdbtk_interp, "enable_external_editor",
-	      enable_external_editor ? "1" : "0", 0);
-  Tcl_SetVar (gdbtk_interp, "external_editor_command",
-	      external_editor_command, 0);
+  if (external_editor_command != NULL)
+    {
+      Tcl_SetVar (gdbtk_interp, "external_editor_command",
+		  external_editor_command, 0);
+      xfree (external_editor_command);
+      external_editor_command = NULL;
+    }
 
   /* close old output and send new to GDBTK */
   ui_file_delete (gdb_stdout);
@@ -500,7 +590,7 @@ gdbtk_init (argv0)
   uiout = cli_out_new (gdb_stdout);
 
 #ifdef __CYGWIN32__
-      (void) FreeConsole ();
+  (void) FreeConsole ();
 #endif
 
   /* find the gdb tcl library and source main.tcl */
@@ -511,7 +601,7 @@ gdbtk_init (argv0)
 proc gdbtk_find_main {} {\n\
     global Paths GDBTK_LIBRARY\n\
     rename gdbtk_find_main {}\n\
-    tcl_findLibrary gdb 1.0 {} main.tcl GDBTK_LIBRARY GDBTK_LIBRARY gdbtk/library gdbtcl {}\n\
+    tcl_findLibrary insight 1.0 {} main.tcl GDBTK_LIBRARY GDBTKLIBRARY\n\
     set Paths(appdir) $GDBTK_LIBRARY\n\
 }\n\
 gdbtk_find_main";
@@ -527,7 +617,7 @@ proc gdbtk_find_main {} {\n\
     } else {\n\
         set debug_startup 0\n\
     }\n\
-    tcl_findLibrary gdb 1.0 {} main.tcl GDBTK_LIBRARY GDBTK_LIBRARY gdbtk/library gdbtcl {} $debug_startup\n\
+    tcl_findLibrary insight 1.0 {} main.tcl GDBTK_LIBRARY GDBTK_LIBRARY\n\
     set Paths(appdir) $GDBTK_LIBRARY\n\
 }\n\
 gdbtk_find_main";
@@ -575,8 +665,7 @@ gdbtk_find_main";
    startup procedure. */
 
 int
-gdbtk_test (filename)
-     char *filename;
+gdbtk_test (char *filename)
 {
   if (access (filename, R_OK) != 0)
     return 0;
@@ -624,15 +713,14 @@ _initialize_gdbtk ()
 }
 
 static void
-tk_command (cmd, from_tty)
-     char *cmd;
-     int from_tty;
+tk_command (char *cmd, int from_tty)
 {
   int retval;
   char *result;
   struct cleanup *old_chain;
 
-  /* Catch case of no argument, since this will make the tcl interpreter dump core. */
+  /* Catch case of no argument, since this will make the tcl interpreter 
+     dump core. */
   if (cmd == NULL)
     error_no_arg ("tcl command to interpret");
 
@@ -649,4 +737,3 @@ tk_command (cmd, from_tty)
 
   do_cleanups (old_chain);
 }
-
