@@ -1,5 +1,5 @@
 # Interface between GDB and Insight.
-# Copyright 1997, 1998, 1999, 2001 Red Hat, Inc.
+# Copyright 1997, 1998, 1999, 2001, 2002 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License (GPL) as published by
@@ -100,7 +100,7 @@ define_hook gdb_trace_find_hook
 # ------------------------------------------------------------------
 
 proc gdbtk_tcl_preloop { } {
-  global gdb_exe_name
+  global gdb_exe_name gdb_current_directory
 
   set_baud
 
@@ -111,14 +111,19 @@ proc gdbtk_tcl_preloop { } {
   after idle gdbtk_idle 
   ManagedWin::startup
 
-  SrcWin::point_to_main
-  set msg ""
-  catch {gdb_cmd "info files"} msg
-  set line1 [string range $msg 0 [string first \n $msg]]  
-  if {[regexp {Symbols from "(.*)"\.} $line1 dummy name]} {
-    set gdb_exe_name $name
+  if {$gdb_exe_name != ""} {
+    # At startup, file_changed_hook is called too late for us, so we
+    # must notice the initial session by hand.  If the arguments exist
+    # -- if the user used `gdb --args' -- then we want the new
+    # arguments and pwd to override what is set in the session.
+    set current_args [gdb_get_inferior_args]
+    set current_dir $gdb_current_directory
+    Session::notice_file_change
+    if {[string length $current_args] > 0} {
+      gdb_set_inferior_args $current_args
+      gdb_cmd "cd $current_dir"
+    }
   }
-
   
   gdbtk_update
 }
@@ -263,7 +268,7 @@ proc gdbtk_cleanup {} {
 
   # Save the session
   if {$gdb_exe_name != ""} {
-    session_save
+    Session::save
   }
 
   # This is a sign that it is too late to be doing updates, etc...
@@ -415,7 +420,7 @@ proc echo {args} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_error {message} {
   if {$::gdbtk_state(console) != ""} {
-    $::gdbtk_state(console) einsert $message err_tag
+    $::gdbtk_state(console) insert $message err_tag
     update
   }
 }
@@ -425,7 +430,7 @@ proc gdbtk_tcl_fputs_error {message} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_log {message} {
   if {$::gdbtk_state(console) != ""} {
-    $::gdbtk_state(console) einsert $message log_tag
+    $::gdbtk_state(console) insert $message log_tag
     update
   }
 }
@@ -435,7 +440,7 @@ proc gdbtk_tcl_fputs_log {message} {
 # ------------------------------------------------------------------
 proc gdbtk_tcl_fputs_target {message} {
   if {$::gdbtk_state(console) != ""} {
-    $::gdbtk_state(console) einsert $message target_tag
+    $::gdbtk_state(console) insert $message target_tag
     update
   }
 }
@@ -509,6 +514,19 @@ proc gdb_run_readline_command {command args} {
   global gdbtk_state
 #  debug "$command $args"
   set gdbtk_state(readlineArgs) $args
+  set gdbtk_state(readlineShowUser) 1
+  gdb_cmd $command
+}
+
+# ------------------------------------------------------------------
+# PROC: gdb_run_readline_command_no_output
+# Run a readline command, but don't show the commands to the user.
+# ------------------------------------------------------------------
+proc gdb_run_readline_command_no_output {command args} {
+  global gdbtk_state
+#  debug "$command $args"
+  set gdbtk_state(readlineArgs) $args
+  set gdbtk_state(readlineShowUser) 0
   gdb_cmd $command
 }
 
@@ -519,7 +537,7 @@ proc gdbtk_tcl_readline_begin {message} {
   global gdbtk_state
 #  debug "readline begin"
   set gdbtk_state(readline) 0
-  if {$gdbtk_state(console) != ""} {
+  if {$gdbtk_state(console) != "" && $gdbtk_state(readlineShowUser)} {
     $gdbtk_state(console) insert $message
   }
 }
@@ -533,7 +551,9 @@ proc gdbtk_tcl_readline {prompt} {
   if {[info exists gdbtk_state(readlineArgs)]} {
     # Not interactive, so pop the list, and print element.
     set cmd [lvarpop gdbtk_state(readlineArgs)]
-    command::insert_command $cmd
+    if {$gdbtk_state(readlineShowUser)} {
+      command::insert_command $cmd
+    }
   } else {
     # Interactive.
 #    debug "interactive"
@@ -712,7 +732,7 @@ proc gdbtk_tcl_file_changed {filename} {
 
   if {$filename == ""} {
     gdb_clear_file
-    run_hooks gdb_clear_file_hook
+    catch {run_hooks gdb_clear_file_hook}
     set ::gdb_exe_name ""
     set ::gdb_loaded 0
     set ::gdb_running 0
@@ -749,10 +769,9 @@ proc gdbtk_tcl_exec_file_display {filename} {
   # quotes, so we need to strip them here.
   # We need to make sure that we turn filename into
   # an absolute path or sessions won't work.
-  set filename [string trim $filename \']
-  if {[string index $filename 0] != "/"} {
-    set pwd [pwd]
-    set filename "$pwd/$filename"
+  if {[file tail $filename] == $filename} {
+    # want full pathname
+    set filename [file join $::gdb_current_directory $filename]
   }
   set_exe_name $filename
   set gdb_exe_changed 0
@@ -816,10 +835,6 @@ proc set_exe_name {exe} {
   global gdb_exe_name gdb_exe_changed
   #debug "exe=$exe  gdb_exe_name=$gdb_exe_name"
 
-  if {$gdb_exe_name != ""} then {
-    session_save
-  }
-
   set gdb_exe_name $exe
   set gdb_exe_changed 1    
 }
@@ -850,9 +865,6 @@ proc set_exe {} {
       tk_messageBox -icon error -default ok \
 	-title "GDB" -type ok -modal system \
 	-message "This executable has no debugging information."
-      set gdb_exe_name {}
-      set file_done 0
-      return
     }
 
     # force new target command
@@ -934,7 +946,7 @@ proc _open_file {{file ""}} {
     gdbtk_idle
     return 0
   }
-  
+
   return 1
 }
 
@@ -958,6 +970,7 @@ proc _close_file {} {
   }
 
   if {$okay} {
+    Session::save
     gdb_clear_file
     gdbtk_tcl_file_changed ""
 
@@ -1024,7 +1037,7 @@ proc set_target_name {{prompt 1}} {
       }
       set hostname [pref getd gdb/load/$target-hostname]
       if {$hostname == ""} {
-	set hostname [pref get gdb/load/default-hostname]
+	set hostname [pref getd gdb/load/default-hostname]
       }
       # replace "com1" with the real port name
       set targ [lrep $targ "com1" $port]
@@ -1091,7 +1104,7 @@ necessary,\nmodify the port setting with the debugger preferences."
     
     if {![catch {pref get gdb/load/$gdb_target_name-after_attaching} aa] && $aa != ""} {
       if {[catch {gdb_cmd $aa} err]} {
-	catch {[ManagedWin::find Console] einsert $err err_tag}
+	catch {[ManagedWin::find Console] insert $err err_tag}
       }
     }
     set gdb_target_changed 0
@@ -1414,7 +1427,7 @@ proc gdbtk_nexti {} {
   catch {gdb_immediate nexti}
 }
 
-  # ------------------------------------------------------------------
+# ------------------------------------------------------------------
 #  PROC: gdbtk_attached
 # ------------------------------------------------------------------
 #
@@ -1669,7 +1682,7 @@ proc gdbtk_clear_file {} {
 
   debug
   # Give widgets a chance to clean up
-  run_hooks gdb_clear_file_hook
+  catch {run_hooks gdb_clear_file_hook}
 
   # Save the target name in case the user has already selected a
   # target. No need to force the user to select it again.
@@ -1718,6 +1731,7 @@ proc initialize_gdbtk {} {
     # Only do this once...
     set gdbtk_state(readline) 0
     set gdbtk_state(console) ""
+    set gdbtk_state(readlineShowUser) 1
   }
 
   # check for existence of a kod command and get it's name and
@@ -1745,3 +1759,9 @@ proc initialize_gdbtk {} {
   set gdb_target_changed 1
 }
 
+# The architecture changed. Inform the UI.
+proc gdbtk_tcl_architecture_changed {} {
+  set e [ArchChangedEvent \#auto]
+  GDBEventHandler::dispatch $e
+  delete object $e
+}
