@@ -52,7 +52,7 @@
 #define	NOT	301
 #define	QID	302
 
-#line 40 "m2-exp.y"
+#line 41 "m2-exp.y"
 
 
 #include "defs.h"
@@ -116,22 +116,17 @@
 #define	YYDEBUG	0		/* Default to no yydebug support */
 #endif
 
-int
-yyparse PARAMS ((void));
+int yyparse (void);
 
-static int
-yylex PARAMS ((void));
+static int yylex (void);
 
-void
-yyerror PARAMS ((char *));
+void yyerror (char *);
 
 #if 0
-static char *
-make_qualname PARAMS ((char *, char *));
+static char *make_qualname (char *, char *);
 #endif
 
-static int
-parse_number PARAMS ((int));
+static int parse_number (int);
 
 /* The sign of the number being parsed. */
 static int number_sign = 1;
@@ -143,7 +138,7 @@ static struct block *modblock=0;
 #endif
 
 
-#line 135 "m2-exp.y"
+#line 131 "m2-exp.y"
 typedef union
   {
     LONGEST lval;
@@ -258,14 +253,14 @@ static const short yyrhs[] = {    70,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   204,   205,   208,   217,   220,   222,   227,   231,   235,   236,
-   239,   243,   247,   251,   255,   261,   267,   271,   277,   281,
-   285,   289,   294,   298,   304,   308,   314,   320,   323,   327,
-   331,   334,   336,   342,   347,   353,   357,   363,   366,   370,
-   375,   380,   385,   391,   397,   405,   409,   413,   417,   421,
-   425,   429,   433,   437,   439,   443,   447,   451,   455,   459,
-   463,   467,   474,   480,   486,   493,   502,   510,   517,   520,
-   527,   534,   538,   547,   559,   567,   571,   587,   638
+   200,   201,   204,   213,   216,   218,   223,   227,   231,   232,
+   235,   239,   243,   247,   251,   257,   263,   267,   273,   277,
+   281,   285,   290,   294,   300,   304,   310,   316,   319,   323,
+   327,   330,   332,   338,   343,   349,   353,   359,   362,   366,
+   371,   376,   381,   387,   393,   401,   405,   409,   413,   417,
+   421,   425,   429,   433,   435,   439,   443,   447,   451,   455,
+   459,   463,   470,   476,   482,   489,   498,   506,   513,   516,
+   523,   530,   534,   543,   555,   563,   567,   583,   634
 };
 #endif
 
@@ -1081,274 +1076,274 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 209 "m2-exp.y"
+#line 205 "m2-exp.y"
 { write_exp_elt_opcode(OP_TYPE);
 		  write_exp_elt_type(yyvsp[0].tval);
 		  write_exp_elt_opcode(OP_TYPE);
 		;
     break;}
 case 4:
-#line 218 "m2-exp.y"
+#line 214 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_IND); ;
     break;}
 case 5:
-#line 221 "m2-exp.y"
+#line 217 "m2-exp.y"
 { number_sign = -1; ;
     break;}
 case 6:
-#line 223 "m2-exp.y"
+#line 219 "m2-exp.y"
 { number_sign = 1;
 			  write_exp_elt_opcode (UNOP_NEG); ;
     break;}
 case 7:
-#line 228 "m2-exp.y"
+#line 224 "m2-exp.y"
 { write_exp_elt_opcode(UNOP_PLUS); ;
     break;}
 case 8:
-#line 232 "m2-exp.y"
+#line 228 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_LOGICAL_NOT); ;
     break;}
 case 11:
-#line 240 "m2-exp.y"
+#line 236 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_CAP); ;
     break;}
 case 12:
-#line 244 "m2-exp.y"
+#line 240 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_ORD); ;
     break;}
 case 13:
-#line 248 "m2-exp.y"
+#line 244 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_ABS); ;
     break;}
 case 14:
-#line 252 "m2-exp.y"
+#line 248 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_HIGH); ;
     break;}
 case 15:
-#line 256 "m2-exp.y"
+#line 252 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_MIN);
 			  write_exp_elt_type (yyvsp[-1].tval);
 			  write_exp_elt_opcode (UNOP_MIN); ;
     break;}
 case 16:
-#line 262 "m2-exp.y"
+#line 258 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_MAX);
 			  write_exp_elt_type (yyvsp[-1].tval);
 			  write_exp_elt_opcode (UNOP_MIN); ;
     break;}
 case 17:
-#line 268 "m2-exp.y"
+#line 264 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_FLOAT); ;
     break;}
 case 18:
-#line 272 "m2-exp.y"
+#line 268 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_VAL);
 			  write_exp_elt_type (yyvsp[-3].tval);
 			  write_exp_elt_opcode (BINOP_VAL); ;
     break;}
 case 19:
-#line 278 "m2-exp.y"
+#line 274 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_CHR); ;
     break;}
 case 20:
-#line 282 "m2-exp.y"
+#line 278 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_ODD); ;
     break;}
 case 21:
-#line 286 "m2-exp.y"
+#line 282 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_TRUNC); ;
     break;}
 case 22:
-#line 290 "m2-exp.y"
+#line 286 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_SIZEOF); ;
     break;}
 case 23:
-#line 295 "m2-exp.y"
+#line 291 "m2-exp.y"
 { write_exp_elt_opcode(UNOP_PREINCREMENT); ;
     break;}
 case 24:
-#line 299 "m2-exp.y"
+#line 295 "m2-exp.y"
 { write_exp_elt_opcode(BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode(BINOP_ADD);
 			  write_exp_elt_opcode(BINOP_ASSIGN_MODIFY); ;
     break;}
 case 25:
-#line 305 "m2-exp.y"
+#line 301 "m2-exp.y"
 { write_exp_elt_opcode(UNOP_PREDECREMENT);;
     break;}
 case 26:
-#line 309 "m2-exp.y"
+#line 305 "m2-exp.y"
 { write_exp_elt_opcode(BINOP_ASSIGN_MODIFY);
 			  write_exp_elt_opcode(BINOP_SUB);
 			  write_exp_elt_opcode(BINOP_ASSIGN_MODIFY); ;
     break;}
 case 27:
-#line 315 "m2-exp.y"
+#line 311 "m2-exp.y"
 { write_exp_elt_opcode (STRUCTOP_STRUCT);
 			  write_exp_string (yyvsp[0].sval);
 			  write_exp_elt_opcode (STRUCTOP_STRUCT); ;
     break;}
 case 29:
-#line 324 "m2-exp.y"
+#line 320 "m2-exp.y"
 { error("Sets are not implemented.");;
     break;}
 case 30:
-#line 328 "m2-exp.y"
+#line 324 "m2-exp.y"
 { error("Sets are not implemented.");;
     break;}
 case 31:
-#line 332 "m2-exp.y"
+#line 328 "m2-exp.y"
 { error("Sets are not implemented.");;
     break;}
 case 32:
-#line 335 "m2-exp.y"
+#line 331 "m2-exp.y"
 { error("Sets are not implemented.");;
     break;}
 case 33:
-#line 337 "m2-exp.y"
+#line 333 "m2-exp.y"
 { error("Sets are not implemented.");;
     break;}
 case 34:
-#line 346 "m2-exp.y"
+#line 342 "m2-exp.y"
 { start_arglist(); ;
     break;}
 case 35:
-#line 348 "m2-exp.y"
+#line 344 "m2-exp.y"
 { write_exp_elt_opcode (MULTI_SUBSCRIPT);
 			  write_exp_elt_longcst ((LONGEST) end_arglist());
 			  write_exp_elt_opcode (MULTI_SUBSCRIPT); ;
     break;}
 case 36:
-#line 356 "m2-exp.y"
+#line 352 "m2-exp.y"
 { start_arglist (); ;
     break;}
 case 37:
-#line 358 "m2-exp.y"
+#line 354 "m2-exp.y"
 { write_exp_elt_opcode (OP_FUNCALL);
 			  write_exp_elt_longcst ((LONGEST) end_arglist ());
 			  write_exp_elt_opcode (OP_FUNCALL); ;
     break;}
 case 39:
-#line 367 "m2-exp.y"
+#line 363 "m2-exp.y"
 { arglist_len = 1; ;
     break;}
 case 40:
-#line 371 "m2-exp.y"
+#line 367 "m2-exp.y"
 { arglist_len++; ;
     break;}
 case 41:
-#line 376 "m2-exp.y"
+#line 372 "m2-exp.y"
 { arglist_len = 1; ;
     break;}
 case 42:
-#line 381 "m2-exp.y"
+#line 377 "m2-exp.y"
 { arglist_len++; ;
     break;}
 case 43:
-#line 386 "m2-exp.y"
+#line 382 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_MEMVAL);
 			  write_exp_elt_type (yyvsp[-2].tval);
 			  write_exp_elt_opcode (UNOP_MEMVAL); ;
     break;}
 case 44:
-#line 392 "m2-exp.y"
+#line 388 "m2-exp.y"
 { write_exp_elt_opcode (UNOP_CAST);
 			  write_exp_elt_type (yyvsp[-3].tval);
 			  write_exp_elt_opcode (UNOP_CAST); ;
     break;}
 case 45:
-#line 398 "m2-exp.y"
+#line 394 "m2-exp.y"
 { ;
     break;}
 case 46:
-#line 406 "m2-exp.y"
+#line 402 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_REPEAT); ;
     break;}
 case 47:
-#line 410 "m2-exp.y"
+#line 406 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_MUL); ;
     break;}
 case 48:
-#line 414 "m2-exp.y"
+#line 410 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_DIV); ;
     break;}
 case 49:
-#line 418 "m2-exp.y"
+#line 414 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_INTDIV); ;
     break;}
 case 50:
-#line 422 "m2-exp.y"
+#line 418 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_REM); ;
     break;}
 case 51:
-#line 426 "m2-exp.y"
+#line 422 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_ADD); ;
     break;}
 case 52:
-#line 430 "m2-exp.y"
+#line 426 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_SUB); ;
     break;}
 case 53:
-#line 434 "m2-exp.y"
+#line 430 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_EQUAL); ;
     break;}
 case 54:
-#line 438 "m2-exp.y"
+#line 434 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_NOTEQUAL); ;
     break;}
 case 55:
-#line 440 "m2-exp.y"
+#line 436 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_NOTEQUAL); ;
     break;}
 case 56:
-#line 444 "m2-exp.y"
+#line 440 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_LEQ); ;
     break;}
 case 57:
-#line 448 "m2-exp.y"
+#line 444 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_GEQ); ;
     break;}
 case 58:
-#line 452 "m2-exp.y"
+#line 448 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_LESS); ;
     break;}
 case 59:
-#line 456 "m2-exp.y"
+#line 452 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_GTR); ;
     break;}
 case 60:
-#line 460 "m2-exp.y"
+#line 456 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_LOGICAL_AND); ;
     break;}
 case 61:
-#line 464 "m2-exp.y"
+#line 460 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_LOGICAL_OR); ;
     break;}
 case 62:
-#line 468 "m2-exp.y"
+#line 464 "m2-exp.y"
 { write_exp_elt_opcode (BINOP_ASSIGN); ;
     break;}
 case 63:
-#line 475 "m2-exp.y"
+#line 471 "m2-exp.y"
 { write_exp_elt_opcode (OP_BOOL);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].ulval);
 			  write_exp_elt_opcode (OP_BOOL); ;
     break;}
 case 64:
-#line 481 "m2-exp.y"
+#line 477 "m2-exp.y"
 { write_exp_elt_opcode (OP_BOOL);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].ulval);
 			  write_exp_elt_opcode (OP_BOOL); ;
     break;}
 case 65:
-#line 487 "m2-exp.y"
+#line 483 "m2-exp.y"
 { write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (builtin_type_m2_int);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].lval);
 			  write_exp_elt_opcode (OP_LONG); ;
     break;}
 case 66:
-#line 494 "m2-exp.y"
+#line 490 "m2-exp.y"
 {
 			  write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (builtin_type_m2_card);
@@ -1357,45 +1352,45 @@ case 66:
 			;
     break;}
 case 67:
-#line 503 "m2-exp.y"
+#line 499 "m2-exp.y"
 { write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (builtin_type_m2_char);
 			  write_exp_elt_longcst ((LONGEST) yyvsp[0].ulval);
 			  write_exp_elt_opcode (OP_LONG); ;
     break;}
 case 68:
-#line 511 "m2-exp.y"
+#line 507 "m2-exp.y"
 { write_exp_elt_opcode (OP_DOUBLE);
 			  write_exp_elt_type (builtin_type_m2_real);
 			  write_exp_elt_dblcst (yyvsp[0].dval);
 			  write_exp_elt_opcode (OP_DOUBLE); ;
     break;}
 case 70:
-#line 521 "m2-exp.y"
+#line 517 "m2-exp.y"
 { write_exp_elt_opcode (OP_LONG);
 			  write_exp_elt_type (builtin_type_int);
 			  write_exp_elt_longcst ((LONGEST) TYPE_LENGTH (yyvsp[-1].tval));
 			  write_exp_elt_opcode (OP_LONG); ;
     break;}
 case 71:
-#line 528 "m2-exp.y"
+#line 524 "m2-exp.y"
 { write_exp_elt_opcode (OP_M2_STRING);
 			  write_exp_string (yyvsp[0].sval);
 			  write_exp_elt_opcode (OP_M2_STRING); ;
     break;}
 case 72:
-#line 535 "m2-exp.y"
+#line 531 "m2-exp.y"
 { yyval.bval = SYMBOL_BLOCK_VALUE(yyvsp[0].sym); ;
     break;}
 case 73:
-#line 539 "m2-exp.y"
+#line 535 "m2-exp.y"
 { struct symbol *sym
 			    = lookup_symbol (copy_name (yyvsp[0].sval), expression_context_block,
 					     VAR_NAMESPACE, 0, NULL);
 			  yyval.sym = sym;;
     break;}
 case 74:
-#line 548 "m2-exp.y"
+#line 544 "m2-exp.y"
 { struct symbol *tem
 			    = lookup_symbol (copy_name (yyvsp[0].sval), yyvsp[-2].bval,
 					     VAR_NAMESPACE, 0, NULL);
@@ -1406,14 +1401,14 @@ case 74:
 			;
     break;}
 case 75:
-#line 560 "m2-exp.y"
+#line 556 "m2-exp.y"
 { write_exp_elt_opcode(OP_VAR_VALUE);
 			  write_exp_elt_block (NULL);
 			  write_exp_elt_sym (yyvsp[0].sym);
 			  write_exp_elt_opcode (OP_VAR_VALUE); ;
     break;}
 case 77:
-#line 572 "m2-exp.y"
+#line 568 "m2-exp.y"
 { struct symbol *sym;
 			  sym = lookup_symbol (copy_name (yyvsp[0].sval), yyvsp[-2].bval,
 					       VAR_NAMESPACE, 0, NULL);
@@ -1428,7 +1423,7 @@ case 77:
 			  write_exp_elt_opcode (OP_VAR_VALUE); ;
     break;}
 case 78:
-#line 588 "m2-exp.y"
+#line 584 "m2-exp.y"
 { struct symbol *sym;
 			  int is_a_field_of_this;
 
@@ -1478,7 +1473,7 @@ case 78:
 			;
     break;}
 case 79:
-#line 639 "m2-exp.y"
+#line 635 "m2-exp.y"
 { yyval.tval = lookup_typename (copy_name (yyvsp[0].sval),
 						expression_context_block, 0); ;
     break;}
@@ -1704,7 +1699,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 644 "m2-exp.y"
+#line 640 "m2-exp.y"
 
 
 #if 0  /* FIXME! */
